/*****************************************************************************
*                                                                            *
*   Paint.NET Toli Effects                                                   *
*   Copyright (C) Toli Cuturicu 2007-2008                                    *
*   e-Mail: tolicuturicu@yahoo.com                                           *
*                                                                            *
*   This program is free software: you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation, either version 3 of the License, or        *
*   (at your option) any later version.                                      *
*                                                                            *
*   This program is distributed in the hope that it will be useful,          *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
*   GNU General Public License for more details.                             *
*                                                                            *
*   You should have received a copy of the GNU General Public License        *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.    *
*                                                                            *
*****************************************************************************/

using System;
using System.Drawing;
using PaintDotNet;
using PaintDotNet.Effects;
using Toli.Properties;

namespace Toli
{
	[EffectCategory(EffectCategory.Effect)]
	public class EngraveEmbossEffect : Effect
	{
		public static string PluginName = IniFile.GetLanguageIniFile().ReadPluginName();

		public EngraveEmbossEffect()
			: base(PluginName, Resources.Icon, SubmenuNames.Stylize, EffectFlags.Configurable) { }

		public override EffectConfigDialog CreateConfigDialog()
		{
			return new EngraveEmbossEffectConfigDialog();
		}

		public override void Render(EffectConfigToken parameters, RenderArgs dstArgs, RenderArgs srcArgs, Rectangle[] rois, int startIndex, int length)
		{
			PdnRegion SelectionRegion = EnvironmentParameters.GetSelection(srcArgs.Surface.Bounds);
			EngraveEmbossEffectConfigToken Token = (EngraveEmbossEffectConfigToken)parameters;
			Rectangle SurfaceRect = srcArgs.Surface.Bounds;
			ColorBgra Pixel;
			int Width = Token.Width;
			double[,] Weights = Token.Weights;
			int Distance = 0;
			int fyStart = 0;
			int fyEnd = 0;
			int fxStart = 0;
			int fxEnd = 0;

			foreach (Rectangle Rect in rois)
				for (int y = Rect.Top; y < Rect.Bottom; y++)
				{
					Distance = y - SurfaceRect.Top;
					if (Distance < Width) fyStart = -Distance;
					else fyStart = -Width;
					Distance = SurfaceRect.Bottom - 1 - y;
					if (Distance < Width) fyEnd = Distance;
					else fyEnd = Width;

					for (int x = Rect.Left; x < Rect.Right; x++)
					{
						Distance = x - SurfaceRect.Left;
						if (Distance < Width) fxStart = -Distance;
						else fxStart = -Width;
						Distance = SurfaceRect.Right - 1 - x;
						if (Distance < Width) fxEnd = Distance;
						else fxEnd = Width;

						double Sum = 0.0;
						for (int fy = fyStart; fy <= fyEnd; fy++)
						{
							for (int fx = fxStart; fx <= fxEnd; fx++)
							{
								double Weight = Weights[fx + Width, fy + Width];
								Pixel = srcArgs.Surface[x + fx, y + fy];
								byte Intensity = Pixel.GetIntensityByte();
								Sum += Weight * (double)Intensity;
							}
						}

						int Value = (int)Sum + byte.MaxValue / 2;
						if (Value > byte.MaxValue) Value = byte.MaxValue;
						else if (Value < 0) Value = 0;
						byte Brga = (byte)Value;
						dstArgs.Surface[x, y] = ColorBgra.FromBgra(Brga, Brga, Brga, byte.MaxValue);
					}
				}
		}

	}
}
