/*****************************************************************************
*                                                                            *
*   Paint.NET Toli Effects                                                   *
*   Copyright (C) Toli Cuturicu 2007-2008                                    *
*   e-Mail: tolicuturicu@yahoo.com                                           *
*                                                                            *
*   This program is free software: you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation, either version 3 of the License, or        *
*   (at your option) any later version.                                      *
*                                                                            *
*   This program is distributed in the hope that it will be useful,          *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
*   GNU General Public License for more details.                             *
*                                                                            *
*   You should have received a copy of the GNU General Public License        *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.    *
*                                                                            *
*****************************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using PaintDotNet;
using PaintDotNet.Effects;

namespace Toli
{
	public partial class EngraveEmbossEffectConfigDialog : EffectConfigDialog
	{
		// Constants
		private const string EffectStyleKey = "EffectStyle";
		private const string EffectWidthKey = "EffectWidth";


		// Constructor
		public EngraveEmbossEffectConfigDialog()
		{
			InitializeComponent();
			Initialize();
		}


		// Fields
		private IniFile LanguageIniFile = IniFile.GetLanguageIniFile();
		private IniFile ConfigurationIniFile = IniFile.GetConfigurationIniFile();


		// Properties
		private int EffectWidth
		{
			get { return this.TrackBarWidth.Value; }
			set { this.TrackBarWidth.Value = value; }
		}

		private EngraveEmbossStyles EffectStyle
		{
			get
			{
				foreach (Control Ctrl in this.Controls)
					if (Ctrl is RadioButton)
						if ((Ctrl as RadioButton).Checked)
							return (EngraveEmbossStyles)Ctrl.TabIndex;
				return EngraveEmbossStyles.Engrave;
			}
			set
			{
				foreach (Control Ctrl in this.Controls)
					if (Ctrl is RadioButton)
						if (Ctrl.TabIndex == (int)value)
							(Ctrl as RadioButton).Checked = true;
			}
		}


		// Methods
		private void Initialize()
		{
			this.Text = (char)32 + EngraveEmbossEffect.PluginName;
			this.LanguageIniFile.ReadControlText(ButtonOK);
			this.LanguageIniFile.ReadControlText(ButtonCancel);
			this.LanguageIniFile.ReadControlText(HeaderLabelWidth);
			this.LanguageIniFile.ReadControlText(HeaderLabelStyle);
			this.LanguageIniFile.ReadControlText(ButtonEngrave);
			this.LanguageIniFile.ReadControlText(ButtonEmboss);
			this.LanguageIniFile.ReadControlText(LabelWidth);
		}


		// Overrides
		protected override void InitialInitToken()
		{
			theEffectToken = new EngraveEmbossEffectConfigToken();
		}

		protected override void InitDialogFromToken(EffectConfigToken effectTokenCopy)
		{
			base.InitDialogFromToken(effectTokenCopy);
			this.EffectWidth = ((EngraveEmbossEffectConfigToken)theEffectToken).Width;
			this.EffectStyle = ((EngraveEmbossEffectConfigToken)theEffectToken).Style;
		}

		protected override void InitTokenFromDialog()
		{
			base.InitTokenFromDialog();
			((EngraveEmbossEffectConfigToken)theEffectToken).Width = this.EffectWidth;
			((EngraveEmbossEffectConfigToken)theEffectToken).Style = this.EffectStyle;
		}


		// Events
		private void Form_Load(object sender, EventArgs e)
		{
			this.ConfigurationIniFile.ReadFormLocation(this);
			this.EffectStyle = (EngraveEmbossStyles)this.ConfigurationIniFile.ReadInteger(EffectStyleKey, (int)EngraveEmbossStyles.Engrave);
			this.EffectWidth = this.ConfigurationIniFile.ReadInteger(EffectWidthKey, 2);
		}

		private void Form_FormClosing(object sender, FormClosingEventArgs e)
		{
			this.ConfigurationIniFile.WriteFormLocation(this);
			this.ConfigurationIniFile.WriteInteger(EffectStyleKey, (int)this.EffectStyle);
			this.ConfigurationIniFile.WriteInteger(EffectWidthKey, this.EffectWidth);
		}

		private void Button_CheckedChanged(object sender, EventArgs e)
		{
			FinishTokenUpdate();
		}

		private void ButtonOK_Click(object sender, EventArgs e)
		{
			UpDownWidth_Leave(sender, e);
			FinishTokenUpdate();
		}

		private void TrackBarWidth_ValueChanged(object sender, System.EventArgs e)
		{
			if (TrackBarWidth.Value != (int)UpDownWidth.Value)
			{
				UpDownWidth.Value = TrackBarWidth.Value;
				FinishTokenUpdate();
			}
		}

		private void UpDownWidth_ValueChanged(object sender, System.EventArgs e)
		{
			if (TrackBarWidth.Value != (int)UpDownWidth.Value)
			{
				TrackBarWidth.Value = (int)UpDownWidth.Value;
				FinishTokenUpdate();
			}
		}

		private void UpDownWidth_Enter(object sender, System.EventArgs e)
		{
			UpDownWidth.Select(0, UpDownWidth.Text.Length);
		}

		private void UpDownWidth_Leave(object sender, System.EventArgs e)
		{
			Utility.ClipNumericUpDown(UpDownWidth);
			if (Utility.CheckNumericUpDown(UpDownWidth))
				UpDownWidth.Value = decimal.Parse(UpDownWidth.Text);
		}

	}
}
