/*****************************************************************************
*                                                                            *
*   Paint.NET Toli Effects                                                   *
*   Copyright (C) Toli Cuturicu 2007-2008                                    *
*   e-Mail: tolicuturicu@yahoo.com                                           *
*                                                                            *
*   This program is free software: you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation, either version 3 of the License, or        *
*   (at your option) any later version.                                      *
*                                                                            *
*   This program is distributed in the hope that it will be useful,          *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
*   GNU General Public License for more details.                             *
*                                                                            *
*   You should have received a copy of the GNU General Public License        *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.    *
*                                                                            *
*****************************************************************************/

using PaintDotNet;
using PaintDotNet.Effects;
using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Toli
{
	public class EngraveEmbossEffectConfigToken : EffectConfigToken
	{
		// Constructors
		public EngraveEmbossEffectConfigToken()
			: base()
		{
			this.Width = 3;
			this.Style = EngraveEmbossStyles.Engrave;
		}

		protected EngraveEmbossEffectConfigToken(EngraveEmbossEffectConfigToken copyMe)
			: base(copyMe)
		{
			this.Width = copyMe.Width;
			this.Style = copyMe.Style;
		}

		public override object Clone()
		{
			return new EngraveEmbossEffectConfigToken(this);
		}


		// Private Fields
		private int _Width;
		private EngraveEmbossStyles _Style;
		private double[,] _Weights = null;


		// Public Properties
		public int Width
		{
			get { return _Width; }
			set { _Width = value; }
		}

		public EngraveEmbossStyles Style
		{
			get { return _Style; }
			set { _Style = value; }
		}

		public double[,] Weights
		{
			get
			{
				double[,] LocalWeights = this._Weights;
				double Value = 0;
				if (LocalWeights == null)
				{
					LocalWeights = new double[2 * Width + 1, 2 * Width + 1];

					for (int I = -Width; (I <= Width); I++)
						for (int J = -Width; (J <= Width); J++)
						{
							switch (this.Style)
							{
								case EngraveEmbossStyles.Engrave:
									Value = Math.Sign(I + J) * Math.Sqrt(Math.Abs(I + J));
									break;
								case EngraveEmbossStyles.Emboss:
									Value = -Math.Sign(I + J) * Math.Sqrt(Math.Abs(I + J));
									break;
								case EngraveEmbossStyles.InvertedEngrave:
									Value = Math.Sign(I - J) * Math.Sqrt(Math.Abs(I - J));
									break;
								case EngraveEmbossStyles.InvertedEmboss:
									Value = -Math.Sign(I - J) * Math.Sqrt(Math.Abs(I - J));
									break;
								default:
									Value = Math.Sign(I + J) * Math.Sqrt(Math.Abs(I + J));
									break;
							}
							LocalWeights[Width + I, Width + J] = Value / (Math.Sqrt(2 * Width) * Width);
						}
					this._Weights = LocalWeights;
				}
				return LocalWeights;
			}
		}

	}
}
