/*****************************************************************************
*                                                                            *
*   Paint.NET Toli Effects                                                   *
*   Copyright (C) Toli Cuturicu 2007-2008                                    *
*   e-Mail: tolicuturicu@yahoo.com                                           *
*                                                                            *
*   This program is free software: you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation, either version 3 of the License, or        *
*   (at your option) any later version.                                      *
*                                                                            *
*   This program is distributed in the hope that it will be useful,          *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
*   GNU General Public License for more details.                             *
*                                                                            *
*   You should have received a copy of the GNU General Public License        *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.    *
*                                                                            *
*****************************************************************************/

using System;
using System.Drawing;
using PaintDotNet;
using PaintDotNet.Effects;
using Toli.Properties;

namespace Toli
{
	[EffectCategory(EffectCategory.Effect)]
	public class MirrorRotateEffect : Effect
	{
		public static string PluginName = IniFile.GetLanguageIniFile().ReadPluginName();

		public MirrorRotateEffect()
			: base(PluginName, Resources.Icon, null, EffectFlags.Configurable) { }

		public override EffectConfigDialog CreateConfigDialog()
		{
			return new MirrorRotateEffectConfigDialog();
		}

		public override void Render(EffectConfigToken parameters, RenderArgs dstArgs, RenderArgs srcArgs, Rectangle[] rois, int startIndex, int length)
		{
			MirrorRotateEffectConfigToken Token = (MirrorRotateEffectConfigToken)parameters;
			PdnRegion SelectionRegion = EnvironmentParameters.GetSelection(srcArgs.Surface.Bounds);
			Rectangle Selection = SelectionRegion.GetBoundsInt();

			// Calculations done PRIOR to the loops
			int w = Selection.Width;
			int h = Selection.Height;
			int d = w - h;
			int x1 = Selection.Left;
			int x2 = Selection.Right - 1;
			int y1 = Selection.Top;
			int y2 = Selection.Bottom - 1;
			int xc1 = x1; int xc2 = x2;
			int yc1 = y1; int yc2 = y2;
			if (d > 0) { xc1 = x1 + d / 2; xc2 = xc1 + h - 1; }
			if (d < 0) { yc1 = y1 - d / 2; yc2 = yc1 + w - 1; }
			int x0 = 0; int y0 = 0;
			int xd = 0; int yd = 0;
			bool Square = (d == 0);

			// The small rectangles loop
			for (int Index = startIndex; Index < startIndex + length; ++Index)
			{
				Rectangle Rect = rois[Index];

				// Switch done PRIOR to the loops; we must have as few instructions in the loops as possible, because they are repeated many times...
				switch (Token.Style)
				{
					case MirrorRotateStyles.None:
						for (int y = Rect.Top; y < Rect.Bottom; y++)
							for (int x = Rect.Left; x < Rect.Right; x++)
								dstArgs.Surface[x, y] = srcArgs.Surface[x, y];
						break;

					case MirrorRotateStyles.Horizontal:
						// No point in repeating the same calculation over and over again
						y0 = y1 + y2;
						for (int y = Rect.Top; y < Rect.Bottom; y++)
							for (int x = Rect.Left; x < Rect.Right; x++)
								dstArgs.Surface[x, y] = srcArgs.Surface[x, y0 - y];
						break;

					case MirrorRotateStyles.Vertical:
						x0 = x1 + x2;
						for (int y = Rect.Top; y < Rect.Bottom; y++)
							for (int x = Rect.Left; x < Rect.Right; x++)
								dstArgs.Surface[x, y] = srcArgs.Surface[x0 - x, y];
						break;

					case MirrorRotateStyles.Rotate180:
						x0 = x1 + x2; y0 = y1 + y2;
						for (int y = Rect.Top; y < Rect.Bottom; y++)
							for (int x = Rect.Left; x < Rect.Right; x++)
								dstArgs.Surface[x, y] = srcArgs.Surface[x0 - x, y0 - y];
						break;

					case MirrorRotateStyles.FirstDiagonal:
						x0 = xc1 + yc2; y0 = yc1 + xc2;
						// No point in testing for square in the loops
						if (Square)
						{
							for (int y = Rect.Top; y < Rect.Bottom; y++)
								for (int x = Rect.Left; x < Rect.Right; x++)
									dstArgs.Surface[x, y] = srcArgs.Surface[x0 - y, y0 - x];
						}
						else
						{
							for (int y = Rect.Top; y < Rect.Bottom; y++)
								for (int x = Rect.Left; x < Rect.Right; x++)
								{
									// We test for Selection only when necessary and do not calculate the coordinates two times
									xd = x0 - y; yd = y0 - x;
									if (Selection.Contains(xd, yd)) dstArgs.Surface[x, y] = srcArgs.Surface[xd, yd];
									else dstArgs.Surface[x, y] = ColorBgra.Transparent;
								}
						}
						break;

					case MirrorRotateStyles.SecondDiagonal:
						x0 = xc2 - yc2; y0 = yc2 - xc2;
						if (Square)
						{
							for (int y = Rect.Top; y < Rect.Bottom; y++)
								for (int x = Rect.Left; x < Rect.Right; x++)
									dstArgs.Surface[x, y] = srcArgs.Surface[x0 + y, y0 + x];
						}
						else
						{
							for (int y = Rect.Top; y < Rect.Bottom; y++)
								for (int x = Rect.Left; x < Rect.Right; x++)
								{
									xd = x0 + y; yd = y0 + x;
									if (Selection.Contains(xd, yd)) dstArgs.Surface[x, y] = srcArgs.Surface[xd, yd];
									else dstArgs.Surface[x, y] = ColorBgra.Transparent;
								}
						}
						break;

					case MirrorRotateStyles.Rotate90:
						x0 = xc2 - yc2; y0 = yc1 + xc2;
						if (Square)
						{
							for (int y = Rect.Top; y < Rect.Bottom; y++)
								for (int x = Rect.Left; x < Rect.Right; x++)
									dstArgs.Surface[x, y] = srcArgs.Surface[x0 + y, y0 - x];
						}
						else
						{
							for (int y = Rect.Top; y < Rect.Bottom; y++)
								for (int x = Rect.Left; x < Rect.Right; x++)
								{
									xd = x0 + y; yd = y0 - x;
									if (Selection.Contains(xd, yd)) dstArgs.Surface[x, y] = srcArgs.Surface[xd, yd];
									else dstArgs.Surface[x, y] = ColorBgra.Transparent;
								}
						}
						break;

					case MirrorRotateStyles.Rotate270:
						x0 = xc1 + yc2; y0 = yc2 - xc2;
						if (Square)
						{
							for (int y = Rect.Top; y < Rect.Bottom; y++)
								for (int x = Rect.Left; x < Rect.Right; x++)
									dstArgs.Surface[x, y] = srcArgs.Surface[x0 - y, y0 + x];
						}
						else
						{
							for (int y = Rect.Top; y < Rect.Bottom; y++)
								for (int x = Rect.Left; x < Rect.Right; x++)
								{
									xd = x0 - y; yd = y0 + x;
									if (Selection.Contains(xd, yd)) dstArgs.Surface[x, y] = srcArgs.Surface[xd, yd];
									else dstArgs.Surface[x, y] = ColorBgra.Transparent;
								}
						}
						break;

					default:
						for (int y = Rect.Top; y < Rect.Bottom; y++)
							for (int x = Rect.Left; x < Rect.Right; x++)
								dstArgs.Surface[x, y] = srcArgs.Surface[x, y];
						break;
				}
				// That is: the code doesn't look elegant, but remember that function calls, tests and calculations take time.
				// This time may not be noticeble outside a loop, but when repeated thousands of times, it starts to matter.
			}
		}

	}
}

/*
None:			[x, y]
Horizontal:		[x, y1 + (y2 - y)]
Vertical:		[x1 + (x2 - x), y]
Rotate180:		[x1 + (x2 - x), y1 + (y2 - y)]
FirstDiagonal:	[x1 + (y2 - y), y1 + (x2 - x)]
SecondDiagonal:	[x2 - (y2 - y), y2 - (x2 - x)]
Rotate90:		[x2 - (y2 - y), y1 + (x2 - x)]
Rotate270:		[x1 + (y2 - y), y2 - (x2 - x)]
*/
