/*****************************************************************************
*                                                                            *
*   Paint.NET Toli Effects                                                   *
*   Copyright (C) Toli Cuturicu 2007-2008                                    *
*   e-Mail: tolicuturicu@yahoo.com                                           *
*                                                                            *
*   This program is free software: you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation, either version 3 of the License, or        *
*   (at your option) any later version.                                      *
*                                                                            *
*   This program is distributed in the hope that it will be useful,          *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
*   GNU General Public License for more details.                             *
*                                                                            *
*   You should have received a copy of the GNU General Public License        *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.    *
*                                                                            *
*****************************************************************************/

using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PaintDotNet.Effects;
using Toli.Properties;

namespace Toli
{
	public partial class MirrorRotateEffectConfigDialog : EffectConfigDialog
	{
		// Constants
		private const int NoOfButtons = 8;
		private const string EffectStyleKey = "EffectStyle";
		private const string ButtonStyleKey = "ButtonStyle";
		private const string ButtonWidthKey = "ButtonWidth";
		private const int DefaultImageOnlyButtonWidthHeight = 56;
		private const int DefaultButtonWidth = 80;
		private const int DefaultButtonHeight = 84;
		private const int DefaultTextOnlyButtonHeight = 36;

		// Constructor
		public MirrorRotateEffectConfigDialog()
		{
			InitializeComponent();
			Initialize();
		}


		// Fields
		private IniFile LanguageIniFile = IniFile.GetLanguageIniFile();
		private IniFile ConfigurationIniFile = IniFile.GetConfigurationIniFile();

		private MirrorRotateStyles EffectStyle;
		private ToolStripItemDisplayStyle ButtonStyle;
		private Image[] ButtonImages;
		private string[] ButtonTexts;
		private int[] ButtonXs;
		private int[] ButtonYs;
		private int ButtonWidth;


		// Overrides
		protected override void InitialInitToken()
		{
			theEffectToken = new MirrorRotateEffectConfigToken();
		}

		protected override void InitTokenFromDialog()
		{
			base.InitTokenFromDialog();
			((MirrorRotateEffectConfigToken)theEffectToken).Style = this.EffectStyle;
		}

		protected override void InitDialogFromToken(EffectConfigToken effectTokenCopy)
		{
			base.InitDialogFromToken(effectTokenCopy);
			this.EffectStyle = ((MirrorRotateEffectConfigToken)theEffectToken).Style;
		}


		// Methods
		private void Initialize()
		{
			this.Text = (char)32 + MirrorRotateEffect.PluginName;
			this.ButtonWidth = this.LanguageIniFile.ReadInteger(ButtonWidthKey, DefaultButtonWidth);
			this.ButtonImages = new Image[NoOfButtons];
			this.ButtonTexts = new string[NoOfButtons];
			this.ButtonXs = new int[NoOfButtons];
			this.ButtonYs = new int[NoOfButtons];
			int Index = 0;
			foreach (Control Ctrl in this.Controls)
				if (Ctrl is Button)
				{
					Index = Ctrl.TabIndex;
					this.LanguageIniFile.ReadControlText(Ctrl);
					this.ButtonTexts[Index] = Ctrl.Text;
					this.ButtonImages[Index] = (Ctrl as Button).Image;
					this.ButtonXs[Index] = Ctrl.Left / Ctrl.Width;
					this.ButtonYs[Index] = Ctrl.Top / Ctrl.Height;
					this.ToolTipProvider.SetToolTip(Ctrl, Ctrl.Text);
				}
		}

		private void SetButtons()
		{
			int Index = 0;
			foreach (Control Ctrl in this.Controls)
				if (Ctrl is Button)
				{
					Index = Ctrl.TabIndex;
					switch (this.ButtonStyle)
					{
						case ToolStripItemDisplayStyle.Image:
							(Ctrl as Button).Image = this.ButtonImages[Index];
							Ctrl.Text = string.Empty;
							Ctrl.Width = DefaultImageOnlyButtonWidthHeight;
							Ctrl.Height = DefaultImageOnlyButtonWidthHeight;
							ToolTipProvider.Active = true;
							break;
						case ToolStripItemDisplayStyle.Text:
							(Ctrl as Button).Image = null;
							Ctrl.Text = this.ButtonTexts[Index];
							Ctrl.Width = ButtonWidth;
							Ctrl.Height = DefaultTextOnlyButtonHeight;
							ToolTipProvider.Active = false;
							break;
						default:
							(Ctrl as Button).Image = this.ButtonImages[Index];
							Ctrl.Text = this.ButtonTexts[Index];
							Ctrl.Width = ButtonWidth;
							Ctrl.Height = DefaultButtonHeight;
							ToolTipProvider.Active = false;
							break;
					}
					Ctrl.Left = this.ButtonXs[Index] * Ctrl.Width;
					Ctrl.Top = this.ButtonYs[Index] * Ctrl.Height;
				}
			int ClientWidth = 2 * this.ButtonNone.Width;
			int ClientHeight = 4 * this.ButtonNone.Height;
			this.ClientSize = new Size(ClientWidth, ClientHeight);
		}


		// Events
		private void Form_Load(object sender, EventArgs e)
		{
			this.ConfigurationIniFile.ReadFormLocation(this);
			this.EffectStyle = (MirrorRotateStyles)this.ConfigurationIniFile.ReadInteger(EffectStyleKey, 0);
			this.ButtonStyle = (ToolStripItemDisplayStyle)this.ConfigurationIniFile.ReadInteger(ButtonStyleKey, (int)ToolStripItemDisplayStyle.ImageAndText);
			SetButtons();
		}

		private void Form_Shown(object sender, EventArgs e)
		{
			foreach (Control Ctrl in this.Controls)
			{
				if (Ctrl.TabIndex == (int)this.EffectStyle)
					Ctrl.Focus();
				Ctrl.Enter += new EventHandler(Button_Enter);
			}
		}

		private void Form_FormClosing(object sender, FormClosingEventArgs e)
		{
			this.ConfigurationIniFile.WriteFormLocation(this);
			this.ConfigurationIniFile.WriteInteger(EffectStyleKey, (int)this.EffectStyle);
			this.ConfigurationIniFile.WriteInteger(ButtonStyleKey, (int)this.ButtonStyle);
		}

		private void Button_Enter(object sender, EventArgs e)
		{
			this.EffectStyle = (MirrorRotateStyles)(sender as Control).TabIndex;
			FinishTokenUpdate();
		}

		private void Button_MouseHover(object sender, EventArgs e)
		{
			(sender as Control).Focus();
		}

		private void Button_Click(object sender, EventArgs e)
		{
			this.EffectStyle = (MirrorRotateStyles)(sender as Control).TabIndex;
			FinishTokenUpdate();
			this.DialogResult = DialogResult.OK;
			Close();
		}

		private void ButtonNone_Click(object sender, EventArgs e)
		{
			this.DialogResult = DialogResult.Cancel;
			Close();
		}

		private void Button_MouseDown(object sender, MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Right)
			{
				switch (this.ButtonStyle)
				{
					case ToolStripItemDisplayStyle.ImageAndText: this.ButtonStyle = ToolStripItemDisplayStyle.Image; break;
					case ToolStripItemDisplayStyle.Image: this.ButtonStyle = ToolStripItemDisplayStyle.Text; break;
					case ToolStripItemDisplayStyle.Text: this.ButtonStyle = ToolStripItemDisplayStyle.ImageAndText; break;
					case ToolStripItemDisplayStyle.None: this.ButtonStyle = ToolStripItemDisplayStyle.ImageAndText; break;
				}
				SetButtons();
			}
		}

	}
}
