/*****************************************************************************
*                                                                            *
*   Paint.NET Toli Effects                                                   *
*   Copyright (C) Toli Cuturicu 2007-2008                                    *
*   e-Mail: tolicuturicu@yahoo.com                                           *
*                                                                            *
*   This program is free software: you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation, either version 3 of the License, or        *
*   (at your option) any later version.                                      *
*                                                                            *
*   This program is distributed in the hope that it will be useful,          *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
*   GNU General Public License for more details.                             *
*                                                                            *
*   You should have received a copy of the GNU General Public License        *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.    *
*                                                                            *
*****************************************************************************/

using System;
using System.Drawing;
using PaintDotNet;
using PaintDotNet.Effects;

namespace Toli
{
	[EffectCategory(EffectCategory.Effect)]
	public class Rotate270Effect : Effect
	{
		public Rotate270Effect()
			: base(ResourceHelper.Rotate270Name, ResourceHelper.Rotate270Icon, ResourceHelper.SubmenuName, EffectFlags.None) { }

		public override void Render(EffectConfigToken parameters, RenderArgs dstArgs, RenderArgs srcArgs, Rectangle[] rois, int startIndex, int length)
		{
			PdnRegion SelectionRegion = EnvironmentParameters.GetSelection(srcArgs.Surface.Bounds);
			Rectangle Selection = SelectionRegion.GetBoundsInt();

			int w = Selection.Width;
			int h = Selection.Height;
			int d = w - h;
			int x1 = Selection.Left;
			int x2 = Selection.Right - 1;
			int y1 = Selection.Top;
			int y2 = Selection.Bottom - 1;
			int xc1 = x1; int xc2 = x2;
			int yc1 = y1; int yc2 = y2;
			if (d > 0) { xc1 = x1 + d / 2; xc2 = xc1 + h - 1; }
			if (d < 0) { yc1 = y1 - d / 2; yc2 = yc1 + w - 1; }
			int x0 = xc1 + yc2;
			int y0 = yc2 - xc2;
			int xd = 0; int yd = 0;
			bool Square = (d == 0);

			for (int Index = startIndex; Index < startIndex + length; ++Index)
			{
				Rectangle Rect = rois[Index];
				if (Square)
				{
					for (int y = Rect.Top; y < Rect.Bottom; y++)
						for (int x = Rect.Left; x < Rect.Right; x++)
							dstArgs.Surface[x, y] = srcArgs.Surface[x0 - y, y0 + x];
				}
				else
				{
					for (int y = Rect.Top; y < Rect.Bottom; y++)
						for (int x = Rect.Left; x < Rect.Right; x++)
						{
							xd = x0 - y; yd = y0 + x;
							if (Selection.Contains(xd, yd)) dstArgs.Surface[x, y] = srcArgs.Surface[xd, yd];
							else dstArgs.Surface[x, y] = ColorBgra.Transparent;
						}
				}
			}
		}

	}
}
