/*****************************************************************************
*                                                                            *
*   Paint.NET Toli Effects                                                   *
*   Copyright (C) Toli Cuturicu 2007-2008                                    *
*   e-Mail: tolicuturicu@yahoo.com                                           *
*                                                                            *
*   This program is free software: you can redistribute it and/or modify     *
*   it under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation, either version 3 of the License, or        *
*   (at your option) any later version.                                      *
*                                                                            *
*   This program is distributed in the hope that it will be useful,          *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
*   GNU General Public License for more details.                             *
*                                                                            *
*   You should have received a copy of the GNU General Public License        *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.    *
*                                                                            *
*****************************************************************************/

using System;
using System.Drawing;
using PaintDotNet;
using PaintDotNet.Effects;
using Toli.Properties;

namespace Toli
{
	[EffectCategory(EffectCategory.Effect)]
	public class SmoothEffect : Effect
	{
		public static string PluginName = IniFile.GetLanguageIniFile().ReadPluginName();

		public SmoothEffect()
			: base(PluginName, Resources.Icon, SubmenuNames.Blurs, EffectFlags.None) { }

		public override void Render(EffectConfigToken parameters, RenderArgs dstArgs, RenderArgs srcArgs, Rectangle[] rois, int startIndex, int length)
		{
			PdnRegion SelectionRegion = EnvironmentParameters.GetSelection(srcArgs.Surface.Bounds);
			ColorBgra Pixel;
			int Count; int Blue; int Green; int Red; int Alpha;
			foreach (Rectangle Rect in rois)
				for (int y = Rect.Top; y < Rect.Bottom; y++)
					for (int x = Rect.Left; x < Rect.Right; x++)
					{
						Count = 0;
						Blue = 0; Green = 0; Red = 0; Alpha = 0;
						for (int j = y - 1; (j <= y + 1); j++)
							for (int i = x - 1; (i <= x + 1); i++)
								if (SelectionRegion.IsVisible(new Point(i, j)))
								{
									Pixel = srcArgs.Surface[i, j];
									Count++;
									Blue += Pixel.B; Green += Pixel.G; Red += Pixel.R; Alpha += Pixel.A;
								}
						Blue /= Count; Green /= Count; Red /= Count; Alpha /= Count;
						Pixel = ColorBgra.FromBgra((byte)Blue, (byte)Green, (byte)Red, (byte)Alpha);
						dstArgs.Surface[x, y] = Pixel;
					}
		}

	}
}
